@@ -37,8 +37,9 @@ typedef struct _authreg_error_st {
 } *authreg_error_t;
 
 /** get a handle for the named module */
-authreg_t authreg_init(c2s_t c2s, char *name) {
-    char mod_fullpath[PATH_MAX], *modules_path;
+authreg_t authreg_init(c2s_t c2s, const char *name) {
+    char mod_fullpath[PATH_MAX];
+    const char *modules_path;
     ar_module_init_fn init_fn = NULL;
     authreg_t ar;
     void *handle;
@@ -120,7 +121,7 @@ void authreg_free(authreg_t ar) {
 }
 
 /** auth logger */
-inline static void _authreg_auth_log(c2s_t c2s, sess_t sess, char *method, char *username, char *resource, int success) {
+inline static void _authreg_auth_log(c2s_t c2s, sess_t sess, const char *method, const char *username, const char *resource, int success) {
     log_write(c2s->log, LOG_NOTICE, "[%d] %s authentication %s: %s@%s/%s %s:%d%s%s",
         sess->s->tag, method, success ? "succeeded" : "failed",
         username, sess->host->realm, resource,
@@ -131,7 +132,7 @@ inline static void _authreg_auth_log(c2s_t c2s, sess_t sess, char *method, char
 
 /** auth get handler */
 static void _authreg_auth_get(c2s_t c2s, sess_t sess, nad_t nad) {
-    int ns, elem, attr;
+    int ns, elem, attr, err;
     char username[1024], id[128];
     int ar_mechs;
 
@@ -165,7 +166,7 @@ static void _authreg_auth_get(c2s_t c2s, sess_t sess, nad_t nad) {
         ar_mechs = ar_mechs | c2s->ar_ssl_mechanisms;
         
     /* no point going on if we have no mechanisms */
-    if(!(ar_mechs & (AR_MECH_TRAD_PLAIN | AR_MECH_TRAD_DIGEST))) {
+    if(!(ar_mechs & (AR_MECH_TRAD_PLAIN | AR_MECH_TRAD_DIGEST | AR_MECH_TRAD_CRAMMD5))) {
         sx_nad_write(sess->s, stanza_tofrom(stanza_error(nad, 0, stanza_err_FORBIDDEN), 0));
         return;
     }
@@ -209,6 +210,19 @@ static void _authreg_auth_get(c2s_t c2s, sess_t sess, nad_t nad) {
     if(ar_mechs & AR_MECH_TRAD_DIGEST && c2s->ar->get_password != NULL)
         nad_append_elem(nad, ns, "digest", 2);
 
+    if (ar_mechs & AR_MECH_TRAD_CRAMMD5 && c2s->ar->create_challenge != NULL) {
+        err = (c2s->ar->create_challenge)(c2s->ar, (char *) username, (char *) sess->auth_challenge, sizeof(sess->auth_challenge));
+        if (0 == err) { /* operation failed */
+            sx_nad_write(sess->s, stanza_tofrom(stanza_error(nad, 0, stanza_err_INTERNAL_SERVER_ERROR), 0));
+            return;
+        }
+        else if (1 == err) { /* operation succeeded */
+            nad_append_elem(nad, ns, "crammd5", 2);
+            nad_append_attr(nad, -1, "challenge", sess->auth_challenge);
+        }
+        else ; /* auth method unsupported for user */
+    }
+
     /* give it back to the client */
     sx_nad_write(sess->s, nad);
 
@@ -270,7 +284,7 @@ static void _authreg_auth_set(c2s_t c2s, sess_t sess, nad_t nad) {
         ar_mechs = ar_mechs | c2s->ar_ssl_mechanisms;
     
     /* no point going on if we have no mechanisms */
-    if(!(ar_mechs & (AR_MECH_TRAD_PLAIN | AR_MECH_TRAD_DIGEST))) {
+    if(!(ar_mechs & (AR_MECH_TRAD_PLAIN | AR_MECH_TRAD_DIGEST | AR_MECH_TRAD_CRAMMD5))) {
         sx_nad_write(sess->s, stanza_tofrom(stanza_error(nad, 0, stanza_err_FORBIDDEN), 0));
         return;
     }
@@ -281,6 +295,24 @@ static void _authreg_auth_set(c2s_t c2s, sess_t sess, nad_t nad) {
         return;
     }
     
+    /* handle CRAM-MD5 response */
+    if(!authd && ar_mechs & AR_MECH_TRAD_CRAMMD5 && c2s->ar->check_response != NULL)
+    {
+        elem = nad_find_elem(nad, 1, ns, "crammd5", 1);
+        if(elem >= 0)
+        {
+            snprintf(str, 1024, "%.*s", NAD_CDATA_L(nad, elem), NAD_CDATA(nad, elem));
+            if((c2s->ar->check_response)(c2s->ar, username, sess->host->realm, sess->auth_challenge, str) == 0)
+            {
+                log_debug(ZONE, "crammd5 auth (check) succeded");
+                authd = 1;
+                _authreg_auth_log(c2s, sess, "traditional.cram-md5", username, resource, TRUE);
+            } else {
+                _authreg_auth_log(c2s, sess, "traditional.cram-md5", username, resource, FALSE);
+            }
+        }
+    }
+
     /* digest auth */
     if(!authd && ar_mechs & AR_MECH_TRAD_DIGEST && c2s->ar->get_password != NULL)
     {
@@ -297,6 +329,8 @@ static void _authreg_auth_set(c2s_t c2s, sess_t sess, nad_t nad) {
                     log_debug(ZONE, "digest auth succeeded");
                     authd = 1;
                     _authreg_auth_log(c2s, sess, "traditional.digest", username, resource, TRUE);
+                } else {
+                    _authreg_auth_log(c2s, sess, "traditional.digest", username, resource, FALSE);
                 }
             }
         }
@@ -313,6 +347,8 @@ static void _authreg_auth_set(c2s_t c2s, sess_t sess, nad_t nad) {
                 log_debug(ZONE, "plaintext auth (compare) succeeded");
                 authd = 1;
                 _authreg_auth_log(c2s, sess, "traditional.plain(compare)", username, resource, TRUE);
+            } else {
+                _authreg_auth_log(c2s, sess, "traditional.plain(compare)", username, resource, FALSE);
             }
         }
     }
@@ -329,6 +365,8 @@ static void _authreg_auth_set(c2s_t c2s, sess_t sess, nad_t nad) {
                 log_debug(ZONE, "plaintext auth (check) succeded");
                 authd = 1;
                 _authreg_auth_log(c2s, sess, "traditional.plain", username, resource, TRUE);
+            } else {
+                _authreg_auth_log(c2s, sess, "traditional.plain", username, resource, FALSE);
             }
         }
     }
