@@ -60,11 +60,11 @@ static char salter[] = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRST
 
 typedef struct mysqlcontext_st {
   MYSQL * conn;
-  char * sql_create;
-  char * sql_select;
-  char * sql_setpassword;
-  char * sql_delete;
-  char * field_password;
+  const char * sql_create;
+  const char * sql_select;
+  const char * sql_setpassword;
+  const char * sql_delete;
+  const char * field_password;
   enum mysql_pws_crypt password_type;
 } *mysqlcontext_t;
 
@@ -86,7 +86,7 @@ static void calc_a1hash(const char *username, const char *realm, const char *pas
 }
 #endif
 
-static MYSQL_RES *_ar_mysql_get_user_tuple(authreg_t ar, char *username, char *realm) {
+static MYSQL_RES *_ar_mysql_get_user_tuple(authreg_t ar, const char *username, const char *realm) {
     mysqlcontext_t ctx = (mysqlcontext_t) ar->private;
     MYSQL *conn = ctx->conn;
     char iuser[MYSQL_LU+1], irealm[MYSQL_LR+1];
@@ -127,7 +127,7 @@ static MYSQL_RES *_ar_mysql_get_user_tuple(authreg_t ar, char *username, char *r
     return res;
 }
 
-static int _ar_mysql_user_exists(authreg_t ar, char *username, char *realm) {
+static int _ar_mysql_user_exists(authreg_t ar, const char *username, const char *realm) {
     MYSQL_RES *res = _ar_mysql_get_user_tuple(ar, username, realm);
 
     if(res != NULL) {
@@ -138,7 +138,7 @@ static int _ar_mysql_user_exists(authreg_t ar, char *username, char *realm) {
     return 0;
 }
 
-static int _ar_mysql_get_password(authreg_t ar, char *username, char *realm, char password[257]) {
+static int _ar_mysql_get_password(authreg_t ar, const char *username, const char *realm, char password[257]) {
     mysqlcontext_t ctx = (mysqlcontext_t) ar->private;
     MYSQL *conn = ctx->conn;
     MYSQL_RES *res = _ar_mysql_get_user_tuple(ar, username, realm);
@@ -175,7 +175,7 @@ static int _ar_mysql_get_password(authreg_t ar, char *username, char *realm, cha
     return 0;
 }
 
-static int _ar_mysql_check_password(authreg_t ar, char *username, char *realm, char password[257]) {
+static int _ar_mysql_check_password(authreg_t ar, const char *username, const char *realm, char password[257]) {
     mysqlcontext_t ctx = (mysqlcontext_t) ar->private;
     char db_pw_value[257];
 #ifdef HAVE_CRYPT
@@ -230,7 +230,7 @@ static int _ar_mysql_check_password(authreg_t ar, char *username, char *realm, c
     return ret;
 }
 
-static int _ar_mysql_set_password(authreg_t ar, char *username, char *realm, char password[257]) {
+static int _ar_mysql_set_password(authreg_t ar, const char *username, const char *realm, char password[257]) {
     mysqlcontext_t ctx = (mysqlcontext_t) ar->private;
     MYSQL *conn = ctx->conn;
     char iuser[MYSQL_LU+1], irealm[MYSQL_LR+1];
@@ -246,13 +246,13 @@ static int _ar_mysql_set_password(authreg_t ar, char *username, char *realm, cha
 
 #ifdef HAVE_CRYPT
     if (ctx->password_type == MPC_CRYPT) {
-       char salt[12] = "$1$";
+       char salt[39] = "$6$rounds=50000$";
        int i;
 
        srand(time(0));
-       for(i=0; i<8; i++)
-               salt[3+i] = salter[rand()%64];
-       salt[11] = '\0';
+       for(i=0; i<22; i++)
+               salt[16+i] = salter[rand()%64];
+       salt[38] = '\0';
        strcpy(password, crypt(password, salt));
     }
 #endif
@@ -281,7 +281,7 @@ static int _ar_mysql_set_password(authreg_t ar, char *username, char *realm, cha
     return 0;
 }
 
-static int _ar_mysql_create_user(authreg_t ar, char *username, char *realm) {
+static int _ar_mysql_create_user(authreg_t ar, const char *username, const char *realm) {
     mysqlcontext_t ctx = (mysqlcontext_t) ar->private;
     MYSQL *conn = ctx->conn;
     char iuser[MYSQL_LU+1], irealm[MYSQL_LR+1];
@@ -318,7 +318,7 @@ static int _ar_mysql_create_user(authreg_t ar, char *username, char *realm) {
     return 0;
 }
 
-static int _ar_mysql_delete_user(authreg_t ar, char *username, char *realm) {
+static int _ar_mysql_delete_user(authreg_t ar, const char *username, const char *realm) {
     mysqlcontext_t ctx = (mysqlcontext_t) ar->private;
     MYSQL *conn = ctx->conn;
     char iuser[MYSQL_LU+1], irealm[MYSQL_LR+1];
@@ -354,16 +354,16 @@ static void _ar_mysql_free(authreg_t ar) {
     if(conn != NULL)
        mysql_close(conn);
 
-    free(ctx->sql_create);
-    free(ctx->sql_select);
-    free(ctx->sql_setpassword);
-    free(ctx->sql_delete);
+    free((void*)ctx->sql_create);
+    free((void*)ctx->sql_select);
+    free((void*)ctx->sql_setpassword);
+    free((void*)ctx->sql_delete);
     free(ctx);
 }
 
 /** Provide a configuration parameter or default value. */
-static char * _ar_mysql_param( config_t c, char * key, char * def ) {
-    char * value = config_get_one( c, key, 0 );
+static const char * _ar_mysql_param( config_t c, const char * key, const char * def ) {
+    const char * value = config_get_one( c, key, 0 );
     if( value == NULL )
       return def;
     else
@@ -375,7 +375,7 @@ static char * _ar_mysql_param( config_t c, char * key, char * def ) {
 /* one each, in order, of the one character sprintf types that are */
 /* expected to follow the escape characters '%' in the template. */
 /* Returns 0 on success, or an error message on failures. */
-static char * _ar_mysql_check_template( char * template, char * types ) {
+static char * _ar_mysql_check_template( const char * template, const char * types ) {
     int pScan = 0;
     int pType = 0;
     char c;
@@ -411,8 +411,8 @@ static char * _ar_mysql_check_template( char * template, char * types ) {
 /* required parameter placeholders.  If there is an error, it is   */
 /* written to the error log. */
 /* Returns 0 on success, or 1 on errors. */
-static int _ar_mysql_check_sql( authreg_t ar, char * sql, char * types ) {
-  char * error;
+static int _ar_mysql_check_sql( authreg_t ar, const char * sql, const char * types ) {
+  const char * error;
 
   error = _ar_mysql_check_template( sql, types );
   if( error == 0 ) return 0;  /* alls right :) */
@@ -424,9 +424,9 @@ static int _ar_mysql_check_sql( authreg_t ar, char * sql, char * types ) {
 
 /** start me up */
 DLLEXPORT int ar_init(authreg_t ar) {
-    char *host, *port, *dbname, *user, *pass;
+    const char *host, *port, *dbname, *user, *pass;
     char *create, *select, *setpassword, *delete;
-    char *table, *username, *realm;
+    const char *table, *username, *realm;
     char *template;
     int strlentur; /* string length of table, user, and realm strings */
     MYSQL *conn;
