@@ -20,24 +20,70 @@
 
 /* this module talks to a PostgreSQL server via libpq */
 
+#define _XOPEN_SOURCE 500
 #include "c2s.h"
 #include <libpq-fe.h>
 
+/* Windows does not have the crypt() function, let's take DES_crypt from OpenSSL instead */
+#if defined(HAVE_OPENSSL_CRYPTO_H) && defined(_WIN32)
+#include <openssl/des.h>
+#define crypt DES_crypt
+#define HAVE_CRYPT 1
+#else
+#ifdef HAVE_CRYPT
+#include <unistd.h>
+#endif
+#endif
+
+#ifdef HAVE_SSL
+/* We use OpenSSL's MD5 routines for the a1hash password type */
+#include <openssl/md5.h>
+#endif
+
 #define PGSQL_LU  1024   /* maximum length of username - should correspond to field length */
 #define PGSQL_LR   256   /* maximum length of realm - should correspond to field length */
 #define PGSQL_LP   256   /* maximum length of password - should correspond to field length */
 
+enum pgsql_pws_crypt {
+    MPC_PLAIN,
+#ifdef HAVE_CRYPT
+    MPC_CRYPT,
+#endif
+#ifdef HAVE_SSL
+    MPC_A1HASH,
+#endif
+};
+
 typedef struct pgsqlcontext_st {
   PGconn * conn;
-  char * sql_create;
-  char * sql_select;
-  char * sql_setpassword;
-  char * sql_delete;
-  char * sql_check_password;
-  char * field_password;
+  const char * sql_create;
+  const char * sql_select;
+  const char * sql_setpassword;
+  const char * sql_delete;
+  const char * sql_check_password;
+  const char * field_password;
+  enum pgsql_pws_crypt password_type;
   } *pgsqlcontext_t;
 
-static PGresult *_ar_pgsql_get_user_tuple(authreg_t ar, char *username, char *realm) {
+#ifdef HAVE_SSL
+static void calc_a1hash(const char *username, const char *realm, const char *password, char *a1hash)
+{
+#define A1PPASS_LEN PGSQL_LU + 1 + PGSQL_LR + 1 + PGSQL_LP + 1 /* user:realm:password\0 */
+    char buf[A1PPASS_LEN];
+    unsigned char md5digest[MD5_DIGEST_LENGTH];
+    int i;
+
+    snprintf(buf, A1PPASS_LEN, "%.*s:%.*s:%.*s", PGSQL_LU, username, PGSQL_LR, realm, PGSQL_LP, password);
+
+    MD5((unsigned char*)buf, strlen(buf), md5digest);
+
+    for(i=0; i<16; i++) {
+        sprintf(a1hash+i*2, "%02hhx", md5digest[i]);
+    }
+}
+#endif
+
+static PGresult *_ar_pgsql_get_user_tuple(authreg_t ar, const char *username, const char *realm) {
     pgsqlcontext_t ctx = (pgsqlcontext_t) ar->private;
     PGconn *conn = ctx->conn;
 
@@ -81,7 +127,7 @@ static PGresult *_ar_pgsql_get_user_tuple(authreg_t ar, char *username, char *re
     return res;
 }
 
-static int _ar_pgsql_user_exists(authreg_t ar, char *username, char *realm) {
+static int _ar_pgsql_user_exists(authreg_t ar, const char *username, const char *realm) {
     if (ar->get_password) {
         PGresult *res = _ar_pgsql_get_user_tuple(ar, username, realm);
 
@@ -96,7 +142,7 @@ static int _ar_pgsql_user_exists(authreg_t ar, char *username, char *realm) {
     }
 }
 
-static int _ar_pgsql_get_password(authreg_t ar, char *username, char *realm, char password[257]) {
+static int _ar_pgsql_get_password(authreg_t ar, const char *username, const char *realm, char password[257]) {
     pgsqlcontext_t ctx = (pgsqlcontext_t) ar->private;
     PGresult *res = _ar_pgsql_get_user_tuple(ar, username, realm);
     int fpass;
@@ -123,7 +169,7 @@ static int _ar_pgsql_get_password(authreg_t ar, char *username, char *realm, cha
     return 0;
 }
 
-static int _ar_pgsql_check_password(authreg_t ar, char *username, char *realm, char password[257])
+static int _ar_pgsql_dbcheck_password(authreg_t ar, const char *username, const char *realm, char password[257])
 {
     pgsqlcontext_t ctx = (pgsqlcontext_t) ar->private;
     PGconn *conn = ctx->conn;
@@ -191,7 +237,63 @@ static int _ar_pgsql_check_password(authreg_t ar, char *username, char *realm, c
     return retval;
 };
 
-static int _ar_pgsql_set_password(authreg_t ar, char *username, char *realm, char password[257]) {
+static int _ar_pgsql_check_password(authreg_t ar, const char *username, const char *realm, char password[257])
+{
+    pgsqlcontext_t ctx = (pgsqlcontext_t) ar->private;
+    char db_pw_value[257];
+#ifdef HAVE_CRYPT
+    char *crypted_pw;
+#endif
+#ifdef HAVE_SSL
+    char a1hash_pw[33];
+#endif
+    int ret;
+
+    ret = _ar_pgsql_get_password(ar, username, realm, db_pw_value);
+    /* return if error */
+    if (ret)
+        return ret;
+
+    switch (ctx->password_type) {
+        case MPC_PLAIN:
+                ret = (strcmp(password, db_pw_value) != 0);
+                break;
+
+#ifdef HAVE_CRYPT
+        case MPC_CRYPT:
+                crypted_pw = crypt(password,db_pw_value);
+                ret = (strcmp(crypted_pw, db_pw_value) != 0);
+                break;
+#endif
+
+#ifdef HAVE_SSL
+        case MPC_A1HASH:
+                if (strchr(username, ':')) {
+                    ret = 1;
+                    log_write(ar->c2s->log, LOG_ERR, "Username cannot contain : with a1hash encryption type.");
+                    break;
+                }
+                if (strchr(realm, ':')) {
+                    ret = 1;
+                    log_write(ar->c2s->log, LOG_ERR, "Realm cannot contain : with a1hash encryption type.");
+                    break;
+                }
+                calc_a1hash(username, realm, password, a1hash_pw);
+                ret = (strncmp(a1hash_pw, db_pw_value, 32) != 0);
+                break;
+#endif
+
+        default:
+        /* should never happen */
+                ret = 1;
+                log_write(ar->c2s->log, LOG_ERR, "Unknown encryption type which passed through config check.");
+                break;
+    }
+
+    return ret;
+}
+
+static int _ar_pgsql_set_password(authreg_t ar, const char *username, const char *realm, char password[257]) {
     pgsqlcontext_t ctx = (pgsqlcontext_t) ar->private;
     PGconn *conn = ctx->conn;
     char iuser[PGSQL_LU+1], irealm[PGSQL_LR+1];
@@ -232,7 +334,7 @@ static int _ar_pgsql_set_password(authreg_t ar, char *username, char *realm, cha
     return 0;
 }
 
-static int _ar_pgsql_create_user(authreg_t ar, char *username, char *realm) {
+static int _ar_pgsql_create_user(authreg_t ar, const char *username, const char *realm) {
     pgsqlcontext_t ctx = (pgsqlcontext_t) ar->private;
     PGconn *conn = ctx->conn;
     char iuser[PGSQL_LU+1], irealm[PGSQL_LR+1];
@@ -281,7 +383,7 @@ static int _ar_pgsql_create_user(authreg_t ar, char *username, char *realm) {
     return 0;
 }
 
-static int _ar_pgsql_delete_user(authreg_t ar, char *username, char *realm) {
+static int _ar_pgsql_delete_user(authreg_t ar, const char *username, const char *realm) {
     pgsqlcontext_t ctx = (pgsqlcontext_t) ar->private;
     PGconn *conn = ctx->conn;
     char iuser[PGSQL_LU+1], irealm[PGSQL_LR+1];
@@ -329,19 +431,19 @@ static void _ar_pgsql_free(authreg_t ar) {
     if(conn != NULL)
        PQfinish(conn);
 
-    free(ctx->sql_create);
-    free(ctx->sql_select);
-    free(ctx->sql_setpassword);
-    free(ctx->sql_delete);
+    free((void*)ctx->sql_create);
+    free((void*)ctx->sql_select);
+    free((void*)ctx->sql_setpassword);
+    free((void*)ctx->sql_delete);
     if (ctx->sql_check_password) {
-        free(ctx->sql_check_password);
+        free((void*)ctx->sql_check_password);
     }
     free(ctx);
 }
 
 /** Provide a configuration parameter or default value. */
-static char * _ar_pgsql_param( config_t c, char * key, char * def ) {
-    char * value = config_get_one( c, key, 0 );
+static const char * _ar_pgsql_param( config_t c, const char * key, const char * def ) {
+    const char * value = config_get_one( c, key, 0 );
     if( value == NULL )
       return def;
     else
@@ -353,7 +455,7 @@ static char * _ar_pgsql_param( config_t c, char * key, char * def ) {
 /* one each, in order, of the one character sprintf types that are */
 /* expected to follow the escape characters '%' in the template. */
 /* Returns 0 on success, or an error message on failures. */
-static char * _ar_pgsql_check_template( char * template, char * types ) {
+static const char * _ar_pgsql_check_template( const char * template, const char * types ) {
     int pScan = 0;
     int pType = 0;
     char c;
@@ -389,8 +491,8 @@ static char * _ar_pgsql_check_template( char * template, char * types ) {
 /* required parameter placeholders.  If there is an error, it is   */
 /* written to the error log. */
 /* Returns 0 on success, or 1 on errors. */
-int _ar_pgsql_check_sql( authreg_t ar, char * sql, char * types ) {
-  char * error;
+int _ar_pgsql_check_sql( authreg_t ar, const char * sql, const char * types ) {
+  const char * error;
 
   error = _ar_pgsql_check_template( sql, types );
   if( error == 0 ) return 0;  /* alls right :) */
@@ -406,9 +508,9 @@ extern int sx_openssl_initialized;
 
 /** start me up */
 int ar_init(authreg_t ar) {
-    char *host, *port, *dbname, *user, *pass, *conninfo;
-    char *create, *select, *setpassword, *delete;
-    char *table, *username, *realm;
+    const char *host, *port, *dbname, *schema, *user, *pass, *conninfo;
+    char *create, *select, *setpassword, *delete, *setsearchpath;
+    const char *table, *username, *realm;
     char *template;
     int strlentur; /* string length of table, user, and realm strings */
     PGconn *conn;
@@ -433,6 +535,21 @@ int ar_init(authreg_t ar) {
 	       , "authreg.pgsql.table"
 	       , "authreg" );
 
+    /* get encryption type used in DB */
+    if (config_get_one(ar->c2s->config, "authreg.pgsql.password_type.plaintext", 0)) {
+        pgsqlcontext->password_type = MPC_PLAIN;
+#ifdef HAVE_CRYPT
+    } else if (config_get_one(ar->c2s->config, "authreg.pgsql.password_type.crypt", 0)) {
+        pgsqlcontext->password_type = MPC_CRYPT;
+#endif
+#ifdef HAVE_SSL
+    } else if (config_get_one(ar->c2s->config, "authreg.pgsql.password_type.a1hash", 0)) {
+        pgsqlcontext->password_type = MPC_A1HASH;
+#endif
+    } else {
+        pgsqlcontext->password_type = MPC_PLAIN;
+    }
+
     /* craft the default SQL statements */
     /* we leave unused statements allocated to simplify code - a small price to pay */
     /* bounds checking and parameter format verification will be perfomed if the statement is used (see next section) */
@@ -485,9 +602,12 @@ int ar_init(authreg_t ar) {
     const char *sql_check_password = _ar_pgsql_param( ar->c2s->config, "authreg.pgsql.sql.checkpassword", 0);
 
     if (sql_check_password) {
+        ar->check_password = _ar_pgsql_dbcheck_password;
         pgsqlcontext->sql_check_password = strdup(sql_check_password);
         if( _ar_pgsql_check_sql( ar, pgsqlcontext->sql_check_password, "sss" ) != 0 ) return 1;
     }
+    else
+        ar->check_password = _ar_pgsql_check_password;
 
     /* echo our configuration to debug */
     log_debug( ZONE, "SQL to create account: %s", pgsqlcontext->sql_create );
@@ -509,6 +629,7 @@ int ar_init(authreg_t ar) {
     host = config_get_one(ar->c2s->config, "authreg.pgsql.host", 0);
     port = config_get_one(ar->c2s->config, "authreg.pgsql.port", 0);
     dbname = config_get_one(ar->c2s->config, "authreg.pgsql.dbname", 0);
+    schema = config_get_one(ar->c2s->config, "authreg.pgsql.schema", 0);
     user = config_get_one(ar->c2s->config, "authreg.pgsql.user", 0);
     pass = config_get_one(ar->c2s->config, "authreg.pgsql.pass", 0);
     conninfo = config_get_one(ar->c2s->config,"authreg.pgsql.conninfo",0);
@@ -531,15 +652,24 @@ int ar_init(authreg_t ar) {
     if(PQstatus(conn) != CONNECTION_OK)
         log_write(ar->c2s->log, LOG_ERR, "pgsql: connection to database failed, will retry later: %s", PQerrorMessage(conn));
 
+    if (schema) {
+        template = "SET search_path TO \"%s\"";
+        setsearchpath = malloc( strlen( template ) + strlen(schema) );
+        sprintf( setsearchpath, template, schema );
+        PQexec(conn, setsearchpath);
+        free(setsearchpath);
+    }
+
     pgsqlcontext->conn = conn;
 
     ar->user_exists = _ar_pgsql_user_exists;
-
-    if (pgsqlcontext->sql_check_password) {
-        ar->check_password = _ar_pgsql_check_password;
-    } else {
+    if (MPC_PLAIN == pgsqlcontext->password_type) {
+        /* only possible with plaintext passwords */
         ar->get_password = _ar_pgsql_get_password;
+    } else {
+        ar->get_password = NULL;
     }
+
     ar->set_password = _ar_pgsql_set_password;
     ar->create_user = _ar_pgsql_create_user;
     ar->delete_user = _ar_pgsql_delete_user;
